''*****************************
''*  ProMD Metal Detector 04.08.07  Christof Eberspcher
''*****************************  
{
  Frequency input at PIN 1 will be measuered.
  There is an active frequency generator (18kHz) connected, which uses an LC-circuit, that will react to metals
  neareby. (Oszill2.doc)
  
  The frequency counter actually gives the time in clock ticks for a given number (MCYCLES) of cycles.
  This gives fast measurement with high resolution.

  A statistics routine will calculate average and standard- deviation of the last NSAMPLES samples.

  The beeper will sound after a significancy- test "Doppelsigma": the value has to be
  consecutively twice out of limits, which are calculated using the average of NSAMPLES samples plus/minus
  a factor SFAK multiplied with the standard- deviation of the samples.

  The display shows:
  1st row: TITEL
  2nd row: Average tics, actual frequency (float)
  3rd row: Difference of the actual tics to the average; Standard- Deviation; something like a Log-value of the dif
  4th: row: bargraph

  Negative values of the difference show nonferrous, positive show steel.   
  
   
}

' Cog usage:
'

'Ports:
'  0: LCD seriell
'  1: Frequenz Eingang
'  6: BEEPER

  
  



CON

  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  x_tiles = 16  '16
  y_tiles = 12  '12

  display= x_tiles*y_tiles*64
  

  '_stack = ($2680 + $2680 + 100) >> 2   'accomodate display memory and stack $3000 bei 16*12 tiles
  '_stack = (2*display + 100) >> 2   'accomodate display memory and stack $3000 bei 16*12 tiles
  _stack = (display + 200) >> 2   'accomodate display memory and stack $3000 bei 16*12 tiles  - no extra buffer

  paramcount = 14
  display_base = $8000- display          '$5000        
  'bitmap_base = display_base- display           '$2000 no extra buffer


  lines = 5
  thickness = 2

  LCD_PIN   = 0                                         ' for Parallax 4x20 serial LCD on P0
  LCD_BAUD  = 19_200
  LCD_LINES = 4

  BEEPER = 6

  RCPIN =       7               'PIN for RC- Input


  MINTIME= 100*5              'Displaywidth >=500 (256samples) in s

  AVGFAK= 0
  KORFAK= 10                    ' Limit drift Kompensation
  MCYCLES= 5000                 ' Cycles to be measured

  NSAMPLES = 30                 ' Number of samples for statistical evaluation
  SFAK = 20                     ' 16 means 1.6
  NOVER = 1                     ' For DoppelSigma number of signal over limits for Significancy
  


VAR

  long  mousex, mousey

  long  tv_status     '0/1/2 = off/visible/invisible           read-only
  long  tv_enable     '0/? = off/on                            write-only
  long  tv_pins       '%ppmmm = pins                           write-only
  long  tv_mode       '%ccinp = chroma,interlace,ntsc/pal,swap write-only
  long  tv_screen     'pointer to screen (words)               write-only
  long  tv_colors     'pointer to colors (longs)               write-only               
  long  tv_hc         'horizontal cells                        write-only
  long  tv_vc         'vertical cells                          write-only
  long  tv_hx         'horizontal cell expansion               write-only
  long  tv_vx         'vertical cell expansion                 write-only
  long  tv_ho         'horizontal offset                       write-only
  long  tv_vo         'vertical offset                         write-only
  long  tv_broadcast  'broadcast frequency (Hz)                write-only
  long  tv_auralcog   'aural fm cog                            write-only

  word  screen[x_tiles * y_tiles]
  long  colors[64]

  byte  x[lines]
  byte  y[lines]
  byte  xs[lines]
  byte  ys[lines]

  long adcin, count, maxin, minin, avg, zoff, ykal, ueff

  'byte  adcbuf[BUFLEN]
     
  long  bufpoint, buftrig, timeus, triglev, repeatoffset, wcount, singlemode, automode, mag  
  long  sawfreq, sqfreq
  long  rcval


  long f1, fhelp ' floats

  long statbuf[NSAMPLES], average, stdev, Einsigma                ' For statistics

  
  

OBJ

  'tv    : "tv"
  'gr    : "graphics"
  
  lcd   : "debug_lcd"

  'num   : "Simple_Numbers"
  fre  : "LowFrequencyCount"
  FM      : "FloatMath"
  FS      : "FloatString"


PUB main | i, aver, freq

    start
    fhelp:=fm.fmul(80_000_000.0,fm.ffloat(MCYCLES))   

    
    i:=0
    repeat
       freq:= fre.getfreq 



       
       i+=1
       if i==3                                ' Nullsetzen Vergleichswert
         aver:=freq << AVGFAK
         AvgStd(freq, 1)                      ' Statistik fllen
       else
         AvgStd(freq,0)  
       aver+= limit(freq-(aver >> AVGFAK))      'Driftkompensation



       lcd.gotoxy(0,1)
       lcd.str(string("                    "))
       lcd.gotoxy(0,2)
       lcd.str(string("                    "))

        if i>3
          lcd.gotoxy(0, 1)
          'lcd.decf(freq, 9)                        ' Messwert anzeigen        
          lcd.decf(average, 9)                        ' Mittelwert anzeigen

          lcd.gotoxy(12,1)
          lcd.str(fs.floattostring(fm.fdiv(fhelp, fm.ffloat(freq)))) 'Frequenz anzeigen

          
          lcd.gotoxy(0, 2)
          'lcd.decf(freq-(aver >> AVGFAK), 8)
          lcd.decf(freq-average, 4)                   ' Abweichung anzeigen

          lcd.gotoxy(7, 2)
          lcd.decf(stdev, 4)                   ' Standardabweichung anzeigen          
              

          lcd.gotoxy(15, 2)
          ' lcd.decf( FindLog((freq-(aver >> AVGFAK))/16) , 3)
          lcd.decf( FindLog( (freq-average)/15) , 3)          

          lcd.gotoxy(0,3)                               ' Bargraph
          lcd.str(string("                    "))
          'lcd.gotoxy(9+FindLog((freq-(aver >> AVGFAK))/16) , 3)
          lcd.gotoxy(9+FindLog((freq-average)/15) , 3)            
          if DoppelSigma(freq)          
             lcd.str(string("#"))
             outa[BEEPER]:=1
          else
             lcd.str(string("*"))
             outa[BEEPER]:=0 
          
       'waitcnt(cnt+30_000_000)

PUB DoppelSigma(wert)
   if Abso(wert-average) > SFAK*stdev/10
     if Einsigma>NOVER
       result:=1
       
     Einsigma+=1
   else
     Einsigma:=0
     result:=0
     

          
PUB Abso(wert)
  if wert<0
     result:= 0-wert
  else
     result:= wert   

       
PUB AvgStd(wert, kontr) | j, sum, dif '

  sum:=0
  
  if kontr==1 ' Fill all
    repeat j from 0 to NSAMPLES-1
      sum += statbuf[j]:= wert
  else    
    repeat j from 1 to NSAMPLES-1
      sum += statbuf[j-1]:= statbuf[j]
    sum += statbuf[NSAMPLES-1]:= wert
      
  average:= sum/NSAMPLES

  sum:=0
  repeat j from 0 to NSAMPLES-1
    dif:= statbuf[j]- average
    sum += dif*dif
    
  stdev:= ^^(sum/NSAMPLES)     
  
     
PUB limit(wert)

  result:= (wert <# KORFAK) #> -KORFAK

  
PUB FindLOG(wert) | j, sign, zwhj
   if wert<0
     wert:= -wert
     sign:= -1
   else
     sign := 1

   zwhj := 0
   repeat j from 0 to 9
     if (1<<j) < wert
        zwhj := j

   result:= sign*zwhj     
            
           

        
PUB start | i, j, k, kk, dx, dy, pp, pq, rr, numx, numchr ' Start und Hauptschleife


  
  dira[BEEPER]~~
  
  lcd.start(LCD_PIN, LCD_BAUD, LCD_LINES)            ' start lcd
  lcd.cursor(0)                                       ' cursor off
  lcd.backLight(true)                                 ' backlight on (if available)
  lcd.cls                                             ' clear the lcd
  lcd.str(string("ProMD2.0 03.08.07CWE", 13))


      
  
  fre.start(MCYCLES)                                     ' Start frequency measurement

  
  'start tv
  longmove(@tv_status, @tvparams, paramcount)
  tv_screen := @screen
  tv_colors := @colors
  'tv.start(@tv_status)

  'init colors
  repeat i from 0 to 63
    'colors[i] := $00001010 * (i+4) & $F + $2B060C02
    'colors[ i ] := $58AE1802 'Black (02), Green (18), Yellow (AE), and Red (58).
    colors[ i ] := $38072B02 '3: 38 rot, 2:07 Weiss, 1:2B blau, 0:02 Black





  
  'init tile screen
  repeat dx from 0 to tv_hc - 1
    repeat dy from 0 to tv_vc - 1
      screen[dy * tv_hc + dx] := display_base >> 6 + dy + dx * tv_vc + ((dy & $3F) << 10)


  'start and setup graphics
  'gr.start
  'gr.setup(x_tiles, y_tiles, 8*x_tiles, 8*y_tiles, bitmap_base)
  'gr.setup(x_tiles, y_tiles, 8*x_tiles, 8*y_tiles, display_base)      'no double buffer
  'gr.colorwidth(3,0)     





   

  'gr.textmode(1,1,6,5)
    'gr.textmode(1,1,6,5) 
  'gr.clear
  'gr.text(-80,+90, string("PropMetDet V1.0 02.08.07 CWE") )
  'gr.copy(display_base)
  








  
  




{PUB Square(freq) | temp ' Square output at SQPIN

    'Configure ctra module. 
  ctrb[30..26] := %00100                     ' Set ctra for "NCO single-ended"
  ctrb[8..0] := SQPIN                           ' Set APIN to SQPIN
  frqb := 0                                  ' Don't send a tone yet.
  dira[SQPIN]~~                                 ' I/O pin to output

    temp := NcoFrqReg(freq)
   
    'Broadcast the signal for 1 s
    frqb := temp
}


PUB NcoFrqReg(frequency) : frqReg
{{
Returns frqReg = frequency  (2  clkfreq) calculated with binary long
division.  This is faster than the floating point library, and takes less
code space.  This method is an adaptation of the CTR object's fraction
method.
}}

  repeat 33                            
    frqReg <<= 1
    if frequency => clkfreq
      frequency -= clkfreq
      frqReg++           
    frequency <<= 1


DAT

tvparams                long    0               'status
                        long    1               'enable
                        long    %001_0101       'pins
                        long    %0000           'mode  0000
                        long    0               'screen
                        long    0               'colors
                        long    x_tiles         'hc
                        long    y_tiles         'vc
                        long    10              'hx
                        long    1               'vx
                        long    0               'ho
                        long    0               'vo
                        long    0               'broadcast
                        long    0               'auralcog